@@ -1,4 +1,4 @@
-#!/usr/share/ucs-test/runner pytest-3 -s -vv
+#!/usr/share/ucs-test/runner pytest-3 -s -vv --log-level=INFO
 ## desc: "Create and delete objects in ucs, check for leftovers or falsely removed objects"
 ## exposure: dangerous
 ## timeout: 7200
@@ -7,8 +7,12 @@
 ## bugs:
 ##  - 50593
 
+# TODO
+# i can't add --log-format '%(asctime)s.%(msecs)s %(levelname)s:%(name)s:%(message)s'
+# to the pytest call, why?
+
+import logging
 import subprocess
-import sys
 import time
 
 import pytest
@@ -26,11 +30,10 @@ from univention.testing.ucs_samba import wait_for_drs_replication
 from univention.testing.udm import verify_udm_object
 from univention.testing.utils import fail, get_ldap_connection
 
-from s4connector import connector_running_on_this_host, connector_setup
+from s4connector import connector_running_on_this_host, connector_setup, wait_for_sync
 
 
-def stderr(msg):
-    print(msg, file=sys.stderr)
+LOGGER = logging.getLogger(__name__)
 
 
 class Users:
@@ -55,6 +58,7 @@ class Users:
         uuid = self.lo.get(userdn, attr=['+'])['entryUUID'][0].decode('UTF-8')
         self.uuids.append(uuid)
         self.users.append((userdn, user))
+        LOGGER.info(f'create user {username} ({uuid})')
 
     def delete_users(self):
         for _dn, user in self.users:
@@ -76,7 +80,7 @@ class Users:
             try:
                 verify_udm_object('users/user', dn, None)
             except AssertionError:
-                stderr('%s still exists in UCS LDAP' % dn)
+                LOGGER.error('%s still exists in UCS LDAP' % dn)
                 return False
         return True
 
@@ -85,7 +89,7 @@ class Users:
             try:
                 verify_udm_object('users/user', dn, {'username': str2dn(dn)[0][0][1]})
             except noObject:
-                stderr('%s does not exist in UCS LDAP' % dn)
+                LOGGER.error('%s does not exist in UCS LDAP' % dn)
                 return False
         return True
 
@@ -93,12 +97,13 @@ class Users:
         db = configdb('/etc/univention/connector/s4internal.sqlite')
         for uuid in self.uuids:
             if db.get('UCS added', uuid):
-                stderr('%s found in UCS added database' % uuid)
+                LOGGER.error('%s found in UCS added database' % uuid)
                 return False
         return True
 
 
 @pytest.mark.skipif(not connector_running_on_this_host(), reason="Univention S4 Connector not configured.")
+@pytest.mark.skipif(configRegistry.is_true("connector/ad/autostart"), reason="This issue is not fixed in the ad connector yet.")
 def test_no_leftovers_after_delete_in_ucs():
     """
     check that all objects are deleted if the (UCS) delete happens during
@@ -111,6 +116,7 @@ def test_no_leftovers_after_delete_in_ucs():
                 <delete (dont delete, different entryUUID)
     object left over
     """
+    wait_for_sync()
     with connector_setup("sync"), UCSTestConfigRegistry():
         # do not update domain users, this changes to timing
         handler_set(['directory/manager/user/primarygroup/update=false'])
@@ -135,8 +141,11 @@ def test_no_leftovers_after_delete_in_ucs():
                 fail("not all users (uid=%s*) have been removed, but should be" % name)
             # we store (ucs) added objects in 'UCS added' and remove the entry during
             # remove to samba, check if the table is clean
-            time.sleep(5)
-            if not user_objects.check_UCS_added_table_is_clean():
+            for i in range(15):
+                time.sleep(10)
+                if user_objects.check_UCS_added_table_is_clean():
+                    break
+            else:
                 fail("some uuid were not removed from s4internal.sqlite->UCS added")
             # check if we really hit the problem (by checking for a specific log message)
             logentry = f'uid={name}{create_users - 1},.* sync ignored: does not exist in UCS but has already been added in the past'
@@ -149,6 +158,7 @@ def test_no_leftovers_after_delete_in_ucs():
 
 
 @pytest.mark.skipif(not connector_running_on_this_host(), reason="Univention S4 Connector not configured.")
+@pytest.mark.skipif(configRegistry.is_true("connector/ad/autostart"), reason="This issue is not fixed in the ad connector yet.")
 def test_do_not_delete_objects_with_different_id():
     '''
     Check if Users in UCS wont be deleted in sync_to_ucs if deleted "by" UCS
