@@ -4,8 +4,8 @@
 ## roles: [domaincontroller_master]
 ## exposure: careful
 ## packages:
-##   - univention-config
-##   - univention-directory-manager-tools
+## - univention-config
+## - univention-directory-manager-tools
 ## timeout: 0
 
 import ipaddress
@@ -19,7 +19,7 @@ import pytest
 from univention.testing import utils
 from univention.testing.decorators import SetTimeout
 from univention.testing.strings import random_name, random_string
-from univention.testing.udm import UCSTestUDM
+from univention.testing.udm import UCSTestUDM, UCSTestUDM_CreateUDMObjectFailed
 
 
 COMPUTER_MODULES = UCSTestUDM.COMPUTER_MODULES
@@ -76,17 +76,184 @@ def get_ssl(name):
     raise LookupError('not found')
 
 
+def verify_dns_objects(computer_name, computer_ip, forward_zone_dn, forward_zone_name, reverse_zone, ip_netmask):
+    """Verify DNS objects which are created alongside a computer object"""
+    if ':' in computer_ip:
+        utils.verify_ldap_object('relativeDomainName=%s,%s' % (
+            computer_name, forward_zone_dn), {'aAAARecord': [computer_ip]}, retry_count=3)
+        relative_domainname = '.'.join(
+            reversed(list(computer_ip.replace(':', '')[int(ip_netmask) // 4:])))
+    else:
+        relative_domainname = '.'.join(
+            computer_ip.split('.')[int(ip_netmask) // 8:])
+        utils.verify_ldap_object('relativeDomainName=%s,%s' % (
+            computer_name, forward_zone_dn), {'aRecord': [computer_ip]}, retry_count=3)
+
+    utils.verify_ldap_object('relativeDomainName=%s,%s' % (relative_domainname, reverse_zone), {
+                             'pTRRecord': ['%s.%s.' % (computer_name, forward_zone_name)]}, retry_count=3)
+
+
 @pytest.mark.roles('domaincontroller_master')
 @pytest.mark.exposure('careful')
 @pytest.mark.parametrize('role', COMPUTER_MODULES)
 class Test_ComputerAllRoles:
-
     @pytest.mark.tags('udm', 'udm-computers', 'apptest')
     def test_all_roles_creation(self, udm, verify_ldap_object, role):
         """Create minimal object for all computer roles"""
         computer = udm.create_object(role, name=random_string())
         verify_ldap_object(computer)
 
+    @pytest.mark.tags('udm', 'udm-computers')
+    @pytest.mark.parametrize(
+        'ip_computer,ip_subnet,ip_network,ip_netmask,ip_range',
+        [
+            ('10.20.30.133', '10.20.30', '10.20.30.0',
+             '24', '10.20.30.2 10.20.30.254'),
+            ('2001:0001:0002:0003:0000:ffff:ffff:1333', '2001:0001:0002:0003',
+             '2001:1:2:3::', '64', '2001:1:2:3::2 2001:1:2:3:0:ffff:ffff:ffff'),
+        ],
+    )
+    def test_all_roles_creation_check_dns(self, ip_computer, ip_subnet, ip_network, ip_netmask, ip_range, udm, ucr, verify_ldap_object, role):
+        """Create object for all computer roles to test if DNS entries are correct"""
+        dNSCn = 'cn=dns,%s' % (ucr.get('ldap/base'),)
+        forwardZoneName = '%s.%s' % (random_name(), random_name())
+        forwardZone = udm.create_object(
+            'dns/forward_zone', zone=forwardZoneName, position=dNSCn, nameserver=random_string(numeric=False))
+        reverseZone = udm.create_object(
+            'dns/reverse_zone', subnet=ip_subnet, position=dNSCn, nameserver=random_string(numeric=False))
+        dhcpService = udm.create_object('dhcp/service', service=random_name())
+
+        networkProperties = {
+            'name': random_name(),
+            'network': ip_network,
+            'netmask': ip_netmask,
+            'dnsEntryZoneForward': forwardZone,
+            'dnsEntryZoneReverse': reverseZone,
+            'dhcpEntryZone': dhcpService,
+            'ipRange': ip_range,
+        }
+        network = udm.create_object('networks/network', **networkProperties)
+        verify_ldap_object(network)
+
+        computerProperties = {'mac': '01:23:45:67:89:ab', 'name': random_name(
+        ), 'ip': ip_computer, 'network': network}
+        computer = udm.create_object(role, **computerProperties)
+        verify_ldap_object(computer)
+
+        verify_dns_objects(computerProperties['name'], computerProperties['ip'],
+                           forwardZone, forwardZoneName, reverseZone, ip_netmask)
+
+    @pytest.mark.tags('udm', 'udm-computers')
+    @pytest.mark.parametrize(
+        'ip_computer,ip_computer_modified,ip_subnet,ip_network,ip_netmask,ip_range',
+        [
+            ('10.20.30.143', '10.20.30.144', '10.20.30',
+             '10.20.30.0', '24', '10.20.30.2 10.20.30.254'),
+            (
+                '2001:0001:0002:0003:0000:ffff:ffff:1343',
+                '2001:0001:0002:0003:0000:ffff:ffff:1344',
+                '2001:0001:0002:0003',
+                '2001:1:2:3::',
+                '64',
+                '2001:1:2:3::2 2001:1:2:3:0:ffff:ffff:ffff',
+            ),
+        ],
+    )
+    def test_all_roles_ip_address_modification(self, ip_computer, ip_computer_modified, ip_subnet, ip_network, ip_netmask, ip_range, udm, ucr, verify_ldap_object, role):
+        """Test if ip address can be modified"""
+        dNSCn = 'cn=dns,%s' % (ucr.get('ldap/base'),)
+        forwardZoneName = '%s.%s' % (random_name(), random_name())
+        forwardZone = udm.create_object(
+            'dns/forward_zone', zone=forwardZoneName, position=dNSCn, nameserver=random_string(numeric=False))
+        reverseZone = udm.create_object(
+            'dns/reverse_zone', subnet=ip_subnet, position=dNSCn, nameserver=random_string(numeric=False))
+        dhcpService = udm.create_object('dhcp/service', service=random_name())
+
+        networkProperties = {
+            'name': random_name(),
+            'network': ip_network,
+            'netmask': ip_netmask,
+            'dnsEntryZoneForward': forwardZone,
+            'dnsEntryZoneReverse': reverseZone,
+            'dhcpEntryZone': dhcpService,
+            'ipRange': ip_range,
+        }
+
+        network = udm.create_object('networks/network', **networkProperties)
+        verify_ldap_object(network)
+
+        computerProperties = {'mac': '01:23:45:67:89:ab', 'name': random_name(
+        ), 'ip': ip_computer, 'network': network}
+        computer = udm.create_object(role, **computerProperties)
+        verify_ldap_object(computer)
+        verify_dns_objects(computerProperties['name'], computerProperties['ip'],
+                           forwardZone, forwardZoneName, reverseZone, ip_netmask)
+
+        computer = udm.modify_object(
+            role, dn=computer, ip=ip_computer_modified)
+        verify_ldap_object(computer)
+        verify_dns_objects(computerProperties['name'], ip_computer_modified,
+                           forwardZone, forwardZoneName, reverseZone, ip_netmask)
+
+    @pytest.mark.tags('udm', 'udm-computers')
+    @pytest.mark.parametrize(
+        'ip_computer,ip_subnet,ip_network,ip_netmask,ip_range',
+        [
+            ('10.20.30.143', '10.20.30', '10.20.30.0',
+             '24', '10.20.30.2 10.20.30.254'),
+            ('2001:0001:0002:0003:0000:ffff:ffff:1343', '2001:0001:0002:0003',
+             '2001:1:2:3::', '64', '2001:1:2:3::2 2001:1:2:3:0:ffff:ffff:ffff'),
+        ],
+    )
+    def test_all_roles_ip_address_double_use(self, ip_computer, ip_subnet, ip_network, ip_netmask, ip_range, udm, ucr, verify_ldap_object, role):
+        """Test if ip address can be modified"""
+        if ":" in ip_computer:
+            pytest.skip("This test fails currently due to Bug #57368. Remove this skip if Bug #57368 is fixed.")
+        dNSCn = 'cn=dns,%s' % (ucr.get('ldap/base'),)
+        forwardZoneName = '%s.%s' % (random_name(), random_name())
+        forwardZone = udm.create_object(
+            'dns/forward_zone', zone=forwardZoneName, position=dNSCn, nameserver=random_string(numeric=False))
+        reverseZone = udm.create_object(
+            'dns/reverse_zone', subnet=ip_subnet, position=dNSCn, nameserver=random_string(numeric=False))
+        dhcpService = udm.create_object('dhcp/service', service=random_name())
+
+        networkProperties = {
+            'name': random_name(),
+            'network': ip_network,
+            'netmask': ip_netmask,
+            'dnsEntryZoneForward': forwardZone,
+            'dnsEntryZoneReverse': reverseZone,
+            'dhcpEntryZone': dhcpService,
+            'ipRange': ip_range,
+        }
+        network = udm.create_object('networks/network', **networkProperties)
+        verify_ldap_object(network)
+
+        computerProperties = {'mac': '01:23:45:67:89:ab', 'name': random_name(
+        ), 'ip': ip_computer, 'network': network}
+        computer = udm.create_object(role, **computerProperties)
+
+        verification_attributes = {'macAddress': [
+            computerProperties['mac']], 'cn': [computerProperties['name']]}
+
+        if ':' in ip_computer:
+            verification_attributes['aAAARecord'] = [ip_computer]
+        else:
+            verification_attributes['aRecord'] = [ip_computer]
+
+        verify_ldap_object(computer, verification_attributes, retry_count=3)
+        verify_dns_objects(computerProperties['name'], computerProperties['ip'],
+                           forwardZone, forwardZoneName, reverseZone, ip_netmask)
+
+        computerProperties_2 = {'mac': '01:23:45:67:89:ac', 'name': random_name(
+        ), 'ip': ip_computer, 'network': network}
+
+        with pytest.raises(UCSTestUDM_CreateUDMObjectFailed) as exc:
+            computer = udm.create_object(role, **computerProperties_2)
+
+        assert f'IP address is already in use: {ip_computer}.' in str(
+            exc.value)
+
     @pytest.mark.tags('udm', 'udm-computers')
     def test_all_roles_removal(self, udm, verify_ldap_object, role):
         """Remove object for all computer roles"""
@@ -415,83 +582,83 @@ class Test_ComputerAllRoles:
             result_ip = lo.getAttr(dn, 'univentionDhcpFixedAddress')[0].decode('ASCII')
             assert result_ip.startswith(NET)
 
-        @pytest.mark.tags('udm', 'udm-computers')
-        def test_nameserver_update_in_zone_on_delete(self, udm, verify_ldap_object, role):
-            """Check if nameservers in forward/reverse DNS zones are updated when deleting the nameserver"""
-            # create zones and computer
-            server_name = 'qwertzu'
-            domain = 'asdfgh'
-            fqdn_dot = server_name + '.' + domain + '.'
-            forward = udm.create_object('dns/forward_zone', zone=domain, nameserver='aaa.aa.')
-            reverse = udm.create_object('dns/reverse_zone', subnet='10.20.30', nameserver='aaa.aa.')
-            computer = udm.create_object(role, set={
-                'ip': '10.20.30.3',
-                'name': server_name,
-                'dnsEntryZoneForward': forward,
-                'dnsEntryZoneReverse': reverse,
-                'domain': domain,
+    @pytest.mark.tags('udm', 'udm-computers')
+    def test_nameserver_update_in_zone_on_delete(self, udm, verify_ldap_object, role):
+        """Check if nameservers in forward/reverse DNS zones are updated when deleting the nameserver"""
+        # create zones and computer
+        server_name = 'qwertzu'
+        domain = 'asdfgh'
+        fqdn_dot = server_name + '.' + domain + '.'
+        forward = udm.create_object('dns/forward_zone', zone=domain, nameserver='aaa.aa.')
+        reverse = udm.create_object('dns/reverse_zone', subnet='10.20.30', nameserver='aaa.aa.')
+        computer = udm.create_object(role, set={
+            'ip': '10.20.30.3',
+            'name': server_name,
+            'dnsEntryZoneForward': forward,
+            'dnsEntryZoneReverse': reverse,
+            'domain': domain,
+        })
+        udm.modify_object('dns/forward_zone', dn=forward, nameserver=[fqdn_dot], wait_for_replication=False)
+        udm.modify_object('dns/reverse_zone', dn=reverse, nameserver=[fqdn_dot], wait_for=True)
+        utils.wait_for_connector_replication()
+        verify_ldap_object(forward, {'nSRecord': ['aaa.aa.', fqdn_dot]})
+        verify_ldap_object(reverse, {'nSRecord': ['aaa.aa.', fqdn_dot]})
+
+        # delete computer and check new name in zones
+        udm.remove_object(role, dn=computer, wait_for=True)
+        utils.wait_for_connector_replication()
+        verify_ldap_object(forward, {'nSRecord': ['aaa.aa.']})
+        verify_ldap_object(reverse, {'nSRecord': ['aaa.aa.']})
+
+    @pytest.mark.tags('udm', 'udm-computers')
+    def test_multiple_dhcp_entry_zones(self, udm, verify_ldap_object, role):
+        """
+        Test appending and removing dhcpEntryZone for all computer roles
+        Computer objects request a uidNumber when creating an object, but the value of
+        univentionLastUsedValue has never been incremented.
+        Since UCS 4.3-2erratumX this is no longer the case and univentionLastUsedValue
+        is never changed.
+        """
+        # bugs: [44937]
+        computerName = random_name()
+
+        service = udm.create_object('dhcp/service', service=random_name())
+        dhcpEntryZones = (
+            [service, '10.20.30.40', '11:11:11:11:11:11'],
+            [service, '10.20.30.41', '22:22:22:22:22:22'],
+            [service, '10.20.30.42', '33:33:33:33:33:33'],
+            [service, '10.20.30.43', '44:44:44:44:44:44'],
+        )
+
+        computer = udm.create_object(role, name=computerName)
+
+        udm.modify_object(role, dn=computer, append={
+            'ip': [zone[1] for zone in dhcpEntryZones],
+            'mac': [zone[2] for zone in dhcpEntryZones],
+            'dhcpEntryZone': [' '.join(zone) for zone in dhcpEntryZones],
+        })
+        for i, (service, ip, mac) in enumerate(dhcpEntryZones, -1):
+            addon = '' if i < 0 else '_uv%d' % (i,)
+            verify_ldap_object('cn=%s%s,%s' % (computerName, addon, service), {
+                'univentionDhcpFixedAddress': [ip],
+                'dhcpHWAddress': ['ethernet %s' % mac],
             })
-            udm.modify_object('dns/forward_zone', dn=forward, nameserver=[fqdn_dot], wait_for_replication=False)
-            udm.modify_object('dns/reverse_zone', dn=reverse, nameserver=[fqdn_dot], wait_for=True)
-            utils.wait_for_connector_replication()
-            verify_ldap_object(forward, {'nSRecord': ['aaa.aa.', fqdn_dot]})
-            verify_ldap_object(reverse, {'nSRecord': ['aaa.aa.', fqdn_dot]})
 
-            # delete computer and check new name in zones
-            udm.remove_object(role, dn=computer, wait_for=True)
-            utils.wait_for_connector_replication()
-            verify_ldap_object(forward, {'nSRecord': ['aaa.aa.']})
-            verify_ldap_object(reverse, {'nSRecord': ['aaa.aa.']})
+        udm.modify_object(role, dn=computer, remove={
+            'ip': [zone[1] for zone in dhcpEntryZones[:2]],
+            'mac': [zone[2] for zone in dhcpEntryZones[:2]],
+            'dhcpEntryZone': [' '.join(zone) for zone in dhcpEntryZones[:2]],
+        })
+        for i, (service, ip, mac) in list(enumerate(dhcpEntryZones, -1))[:2]:
+            addon = '' if i < 0 else '_uv%d' % (i,)
+            verify_ldap_object('cn=%s%s,%s' % (ldap.dn.escape_dn_chars(computerName), addon, service), should_exist=False)
 
-        @pytest.mark.tags('udm', 'udm-computers')
-        def test_multiple_dhcp_entry_zones(self, udm, verify_ldap_object, role):
-            """
-            Test appending and removing dhcpEntryZone for all computer roles
-            Computer objects request a uidNumber when creating an object, but the value of
-            univentionLastUsedValue has never been incremented.
-            Since UCS 4.3-2erratumX this is no longer the case and univentionLastUsedValue
-            is never changed.
-            """
-            # bugs: [44937]
-            computerName = random_name()
-
-            service = udm.create_object('dhcp/service', service=random_name())
-            dhcpEntryZones = (
-                [service, '10.20.30.40', '11:11:11:11:11:11'],
-                [service, '10.20.30.41', '22:22:22:22:22:22'],
-                [service, '10.20.30.42', '33:33:33:33:33:33'],
-                [service, '10.20.30.43', '44:44:44:44:44:44'],
-            )
-
-            computer = udm.create_object(role, name=computerName)
-
-            udm.modify_object(role, dn=computer, append={
-                'ip': [zone[1] for zone in dhcpEntryZones],
-                'mac': [zone[2] for zone in dhcpEntryZones],
-                'dhcpEntryZone': [' '.join(zone) for zone in dhcpEntryZones],
+        for i, (service, ip, mac) in list(enumerate(dhcpEntryZones, -1))[2:]:
+            addon = '' if i < 0 else '_uv%d' % (i,)
+            verify_ldap_object('cn=%s%s,%s' % (ldap.dn.escape_dn_chars(computerName), addon, service), {
+                'univentionDhcpFixedAddress': [ip],
+                'dhcpHWAddress': ['ethernet %s' % mac],
             })
-            for i, (service, ip, mac) in enumerate(dhcpEntryZones, -1):
-                addon = '' if i < 0 else '_uv%d' % (i,)
-                verify_ldap_object('cn=%s%s,%s' % (computerName, addon, service), {
-                    'univentionDhcpFixedAddress': [ip],
-                    'dhcpHWAddress': ['ethernet %s' % mac],
-                })
-
-            udm.modify_object(role, dn=computer, remove={
-                'ip': [zone[1] for zone in dhcpEntryZones[:2]],
-                'mac': [zone[2] for zone in dhcpEntryZones[:2]],
-                'dhcpEntryZone': [' '.join(zone) for zone in dhcpEntryZones[:2]],
-            })
-            for i, (service, ip, mac) in list(enumerate(dhcpEntryZones, -1))[:2]:
-                addon = '' if i < 0 else '_uv%d' % (i,)
-                verify_ldap_object('cn=%s%s,%s' % (ldap.dn.escape_dn_chars(computerName), addon, service), should_exist=False)
-
-            for i, (service, ip, mac) in list(enumerate(dhcpEntryZones, -1))[2:]:
-                addon = '' if i < 0 else '_uv%d' % (i,)
-                verify_ldap_object('cn=%s%s,%s' % (ldap.dn.escape_dn_chars(computerName), addon, service), {
-                    'univentionDhcpFixedAddress': [ip],
-                    'dhcpHWAddress': ['ethernet %s' % mac],
-                })
 
     @pytest.mark.tags('udm', 'udm-computers', 'apptest')
     def test_all_roles_univentionLastUsedValue(self, udm, ucr, lo, verify_ldap_object, wait_for_replication_cleanup, role):
