@@ -30,22 +30,27 @@
 
 from __future__ import annotations
 
+import copy
+import functools
+import json
 import os
+import socket
 from types import SimpleNamespace
-from typing import Callable, Iterator
+from typing import Callable, Generator, Iterator
 
 import pytest
 from keycloak import KeycloakAdmin, KeycloakOpenID
-from playwright.sync_api import Page, expect
+from playwright.sync_api import Browser, BrowserContext, Page, expect
 from utils import (
-    get_portal_tile, keycloak_login, keycloak_password_change, legacy_auth_config_create, legacy_auth_config_remove,
-    run_command,
+    get_portal_tile, grant_oidc_privileges, keycloak_login, keycloak_password_change, legacy_auth_config_create,
+    legacy_auth_config_remove, run_command,
 )
 
 from univention.appcenter.actions import get_action
 from univention.appcenter.app_cache import Apps
 from univention.config_registry import ConfigRegistry
 from univention.lib.misc import custom_groupname
+from univention.testing.pytest_univention_playwright import fixtures
 from univention.testing.udm import UCSTestUDM
 from univention.testing.utils import UCSTestDomainAdminCredentials, get_ldap_connection, wait_for_listener_replication
 from univention.udm import UDM
@@ -78,6 +83,15 @@ def keycloak_secret() -> str | None:
     return password
 
 
+@pytest.fixture(autouse=True)
+def is_keycloak(request, keycloak_config, ucr_proper):
+    if request.node.get_closest_marker('is_keycloak'):
+        keycloak_ip = socket.gethostbyname(keycloak_config.server)
+        my_ip = socket.gethostbyname(ucr_proper['hostname'])
+        if keycloak_ip != my_ip:
+            pytest.skip('this system is not the keycloak server, test makes no sense here')
+
+
 @pytest.fixture()
 def keycloak_admin() -> str:
     return 'admin'
@@ -179,6 +193,7 @@ def portal_config(ucr_proper: ConfigRegistry) -> SimpleNamespace:
         'title': 'Univention Portal',
         'sso_login_tile': 'Login (Single sign-on)',
         'sso_login_tile_de': 'Anmelden (Single Sign-on)',
+        'sso_oidc_login_tile': 'OIDC Login',
         'tile_name_class': 'portal-tile__name',
         'category_title_class': 'portal-category__title',
         'categories_id': 'portalCategories',
@@ -208,6 +223,7 @@ def keycloak_config(ucr_proper: ConfigRegistry) -> SimpleNamespace:
         'url': url,
         'admin_url': f'{url}/admin',
         'token_url': f'{url}/realms/ucs/protocol/openid-connect/token',
+        'logout_url': f'{url}/realms/ucs/protocol/openid-connect/logout',
         'master_token_url': f'{url}/realms/master/protocol/openid-connect/token',
         'users_url': f'{url}/admin/realms/ucs/users',
         'client_session_stats_url': f'{url}/admin/realms/ucs/client-session-stats',
@@ -257,29 +273,37 @@ def browser_type_launch_args(browser_type_launch_args):
     }
 
 
-@pytest.fixture()
-def portal_login_via_keycloak(page: Page, portal_config: SimpleNamespace, keycloak_config: SimpleNamespace):
-    def _func(
-        username: str,
-        password: str,
-        fails_with: str | None = None,
-        new_password: str | None = None,
-        new_password_confirm: str | None = None,
-        verify_login: bool | None = True,
-        url: str | None = portal_config.url,
-        no_login: bool = False,
-    ):
+def __portal_login_func(
+    portal_config: SimpleNamespace,
+    keycloak_config: SimpleNamespace,
+    page: Page,
+    username: str,
+    password: str,
+    fails_with: str | None = None,
+    new_password: str | None = None,
+    new_password_confirm: str | None = None,
+    verify_login: bool | None = True,
+    url: str | None = None,
+    no_login: bool = False,
+    protocol: str | None = 'saml',
+):
+    try:
+        url = url or portal_config.url
         page.goto(url)
         expect(page).to_have_title(portal_config.title)
         lang = page.evaluate('() => window.navigator.userLanguage || window.navigator.language')
         sso_login_tile = portal_config.sso_login_tile if lang == 'en-US' else portal_config.sso_login_tile_de
+        if protocol == 'oidc':
+            sso_login_tile = portal_config.sso_oidc_login_tile
         get_portal_tile(page, sso_login_tile, portal_config).click()
         # login
         keycloak_login(page, keycloak_config, username, password, fails_with=fails_with if not new_password else None, no_login=no_login)
         # check password change
         if new_password:
-            new_password_confirm = new_password_confirm if new_password_confirm else new_password
-            keycloak_password_change(page, keycloak_config, password, new_password, new_password_confirm, fails_with=fails_with)
+            new_password_confirm = new_password_confirm or new_password
+            keycloak_password_change(page, keycloak_config, username, password, new_password, new_password_confirm, fails_with=fails_with)
+        if protocol == 'oidc':
+            grant_oidc_privileges(page)
         if fails_with or no_login:
             return page
         # check that we are logged in
@@ -287,8 +311,19 @@ def portal_login_via_keycloak(page: Page, portal_config: SimpleNamespace, keyclo
             header_menu = page.locator(f'#{portal_config.header_menu_id}')
             expect(header_menu, 'header menu not visible').to_be_visible()
         return page
+    except Exception:
+        print(page.content())
+        raise
 
-    return _func
+
+@pytest.fixture()
+def portal_login_via_keycloak(page: Page, portal_config: SimpleNamespace, keycloak_config: SimpleNamespace):
+    return functools.partial(__portal_login_func, portal_config, keycloak_config, page)
+
+
+@pytest.fixture()
+def portal_login_via_keycloak_custom_page(portal_config: SimpleNamespace, keycloak_config: SimpleNamespace):
+    return functools.partial(__portal_login_func, portal_config, keycloak_config)
 
 
 @pytest.fixture()
@@ -300,13 +335,19 @@ def keycloak_adm_login(page: Page, keycloak_config: SimpleNamespace):
         url: str | None = keycloak_config.url,
         no_login: bool = False,
     ):
-        page.goto(url)
-        expect(page).to_have_title('Univention Corporate Server Single-Sign On')
-        keycloak_login(page, keycloak_config, username, password, fails_with=fails_with, no_login=no_login)
-        # check that we are logged in
-        if not fails_with or not no_login:
-            return page
-        expect(page).to_have_title('Keycloak Administration UI')
+        try:
+            page.goto(url)
+            page.wait_for_load_state('networkidle', timeout=5 * 1000)
+            print(page.content())
+            expect(page).to_have_title('Univention Corporate Server Single-Sign On', timeout=30 * 1000)
+            keycloak_login(page, keycloak_config, username, password, fails_with=fails_with, no_login=no_login)
+            # check that we are logged in
+            if fails_with or no_login:
+                return page
+            expect(page).to_have_title('Keycloak Administration Console')
+        except Exception:
+            print(page.content())
+            raise
         return page
 
     return _func
@@ -443,3 +484,38 @@ def legacy_authorization_setup_oidc(
         run_command(['univention-keycloak', 'legacy-authentication-flow', 'delete', '--flow', 'direct grant'])
         legacy_auth_config_remove(keycloak_administrator_connection, groups)
         keycloak_administrator_connection.delete_client(client_id)
+
+
+@pytest.fixture()
+def oidc_client_logout_meachanism(request):
+    if request.param == 'frontchannel':
+        pytest.skip("frontchannel logout doesn't currently does not work for external OPs. Skipp all frontchannel logout tests for now.")
+    modified_clients = []
+    oidc_clients = json.loads(run_command(['univention-keycloak', 'oidc/rp', 'get', '--json', '--all']))
+
+    for client in oidc_clients:
+        if '/univention/oidc' not in client['clientId']:
+            continue
+        modified_clients.append(client)
+        oidc_client_updated = copy.deepcopy(client)
+        oidc_client_updated['frontchannelLogout'] = request.param == 'frontchannel'
+        oidc_client_updated = json.dumps(oidc_client_updated)
+        run_command(['univention-keycloak', 'oidc/rp', 'update', client['clientId'], oidc_client_updated])
+
+    yield request.param
+
+    for client in modified_clients:
+        run_command(['univention-keycloak', 'oidc/rp', 'update', client['clientId'], json.dumps(client)])
+
+
+@pytest.fixture()
+def multi_tab_context(browser: Browser, request: pytest.FixtureRequest, ucr_proper) -> Generator[BrowserContext, None, None]:
+    context = browser.new_context(ignore_https_errors=True)
+    context.set_default_timeout(30 * 1000)
+    expect.set_options(timeout=30 * 1000)
+
+    context.tracing.start(screenshots=True, snapshots=True, sources=True)
+
+    yield context
+
+    fixtures.teardown_umc_browser_test(request, ucr_proper, context.pages, context, browser)
