@@ -37,6 +37,8 @@ import requests
 from keycloak import KeycloakAdmin
 from playwright.sync_api import Page, expect
 
+from univention.testing.utils import wait_for_listener_replication_and_postrun
+
 
 def host_is_alive(host: str) -> bool:
     command = ['ping', '-c', '2', host]
@@ -50,6 +52,7 @@ def get_portal_tile(page: Page, text: str, portal_config: SimpleNamespace):
 def keycloak_password_change(
     page: Page,
     keycloak_config: SimpleNamespace,
+    username: str,
     password: str,
     new_password: str,
     new_password_confirm: str,
@@ -62,7 +65,12 @@ def keycloak_password_change(
 
     if fails_with:
         error = page.locator(keycloak_config.password_update_error_css_selector.replace("[class='", ".").replace("']", "").replace(" ", "."))
-        assert fails_with == error.inner_text(), f'{fails_with} != {error.inner_text()}'
+        assert fails_with in error.inner_text(), f'{fails_with} != {error.inner_text()}'
+        return
+
+    wait_for_listener_replication_and_postrun()
+    if (page.get_by_label("Username or email").is_visible()):
+        keycloak_login(page=page, username=username, password=new_password, keycloak_config=keycloak_config)
 
 
 def keycloak_auth_header(config: SimpleNamespace) -> dict:
@@ -106,6 +114,21 @@ def keycloak_sessions_by_user(config: SimpleNamespace, username: str) -> dict:
     return response.json()
 
 
+def grant_oidc_privileges(page: Page) -> None:
+    if "oidc" in page.url:
+        try:
+            accept = page.get_by_role("button", name="Yes")
+            expect(accept, "button accept to grant privileges not visible").to_be_visible()
+            accept.click()
+        except AssertionError:
+            pass
+
+
+def portal_logout(page: Page, portal_config: SimpleNamespace) -> None:
+    page.click(f"[id={portal_config.header_menu_id}]")
+    page.click(f"[id={portal_config.logout_button_id}]")
+
+
 def keycloak_login(
     page: Page,
     keycloak_config: SimpleNamespace,
